-------------------------------------------------
--  In   : CLK      20KHz
--       : RESET    Active Hi 
--  Out  : Tick100usOut
--       : Tick1msOut 
--       : Tick10msOut 
-------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity TIMER is
	port( RESET : in std_logic; 
		CLK : in std_logic;
		Tick100usOut : out std_logic;
		Tick1msOut : out std_logic;
		Tick10msOut : out std_logic
	);
end TIMER;

architecture RTL of TIMER is

signal TickerCounterLow : std_logic;
signal TickerCounterMid : std_logic_vector( 3 downto 0 );
signal TickerCounterHi : std_logic_vector( 3 downto 0 );
signal TickerCounterMidCarry : std_logic;
signal TickerCounterHiCarry : std_logic;

begin

Tick100usOut <= TickerCounterLow;
Tick1msOut <= TickerCounterMidCarry;
Tick10msOut <= TickerCounterHiCarry;

	--
	--100us Counter
	--
	process( CLK, RESET ) begin
 		if ( RESET = '1' ) then
			TickerCounterLow <= '0';
		elsif ( CLK'event and CLK = '1' ) then
			TickerCounterLow <= not TickerCounterLow;
		end if;
	end process;

	--
	--1ms Counter
	--
	process( CLK, RESET ) begin
 		if ( RESET = '1' ) then
			TickerCounterMid <= "0000";
		elsif ( CLK'event and CLK = '1' ) then
			if ( TickerCounterLow = '1' ) then
				if ( TickerCounterMidCarry = '1' ) then
					TickerCounterMid <= "0000";
				else
					TickerCounterMid <= TickerCounterMid + 1;
				end if;
			end if;
		end if;
	end process;

	--
	--1ms Counter Carry Generate
	--
	process( TickerCounterLow, TickerCounterMid ) begin
		if ( TickerCounterLow = '1' and  TickerCounterMid = "1001" ) then
			 TickerCounterMidCarry <= '1';
		else
			 TickerCounterMidCarry <= '0';
		end if;
	end process;

	--
	--10ms Counter
	--
	process( CLK, RESET ) begin
 		if ( RESET = '1' ) then
			TickerCounterHi <= "0000";
		elsif ( CLK'event and CLK = '1' ) then
			if ( TickerCounterMidCarry = '1' ) then
				if ( TickerCounterHiCarry = '1' ) then
					TickerCounterHi <= "0000";
				else
					TickerCounterHi <= TickerCounterHi + 1;
				end if;
			end if;
		end if;
	end process;

	--
	--10ms Counter Carry Generate
	--
	process( TickerCounterMidCarry, TickerCounterHi ) begin
		if ( TickerCounterMidCarry = '1' and  TickerCounterHi = "1001" ) then
			 TickerCounterHiCarry <= '1';
		else
			 TickerCounterHiCarry <= '0';
		end if;
	end process;

end RTL;

